#!/bin/bash

KLIST="1 2 3 4 5 6 7 8 9 10 25 50 75 100"

run_eval() {
	f=$1
	pattern=$2
	ff=$(basename $f)
	fname="${ff%.*}"
	ddir=$(dirname $f)

	summaries=$ddir"/summaries/"
	if [ ! -d $summaries ]; then
		mkdir $summaries
	fi

	if [ ! -d $ddir"/output" ]; then
	    mkdir $ddir"/output"
	fi

	summary="${ddir}/summaries/ilp_${fname}_summary.csv"
	summaryp="${ddir}/summaries/pclustering_${fname}_summary.csv"

	if [ -e $pattern ]; then
		pp=$(basename $pattern)
		pname="${pp%.*}"
		summary="${ddir}/summaries/ilp_${fname}_${pname}_summary.csv"
		summaryp="${ddir}/summaries/pclustering_${fname}_${pname}_summary.csv"
		patternArg="--p=${pattern}"
		patternArgIlp="-p ${pattern}"
	else
		patternArg=""
		patternArgIlp=""
	fi

	for k in $KLIST
	do

		if [ -e $pattern ]; then
			output="${ddir}/output/${fname}_k=${k}_${pname}_ilpanonymized.csv"
			outputp="${ddir}/output/${fname}_k=${k}_${pname}.csv"
		else
			output="${ddir}/output/${fname}_k=${k}_ilpanonymized.csv"
			outputp="${ddir}/output/${fname}_k=${k}.csv"
		fi

		if [ -e $output ]; then
			echo "Logfile \"$output\" already exists. Skipping that instance.";
		else
				./patternILP/patternILP $f -k $k -d $f".names" $patternArgIlp > ${output}.log
				mv $f".ilpanonymized.csv" $output
				ilpline=$(head -n 1 $f".ilpresult")
				echo $fname"," $k"," $ilpline >> $summary
				rm -rf $f".ilpresult"
		fi

		if [ -e $outputp ]; then
			echo "Logfile \"$outputp\" already exists. Skipping that instance.";
		else
			echo ""
			echo "starting pclustering"
			(./pclustering $f --k=$k --colTypes=$f".names" $patternArg) > prog_out.txt
			mv "output/${fname}_k=${k}.csv" $outputp
			pclusteringline=$(grep "#" prog_out.txt | cut -d "#" -f 2)
			echo "finished pclustering: "$fname $pclusteringline
			echo $fname $pclusteringline >> $summaryp
			rm -rf prog_out.txt
		fi
		#rm -rf output
	done
}

run_eval "./datasets/nursery/nursery_8_12960.csv" "no-pattern"
run_eval "./datasets/cmc/cmc.csv" "./datasets/cmc/cmcPatterns/cmcPattern2.csv"
run_eval "./datasets/adult/adult_9_Machanavajjhala.csv" "./datasets/adult/adult_9_MachanavajjhalaPatterns/adult9patterns.csv"
